import LinesAndColumns from "lines-and-columns";
export default class BaseParser {
    hasPlugin(name) {
        return Boolean(this.plugins[name]);
    }
    // This function is used to raise exceptions on parse errors. It
    // takes an offset integer (into the current `input`) to indicate
    // the location of the error, attaches the position to the end
    // of the error message, and then raises a `SyntaxError` with that
    // message.
    raise(pos, message, missingPluginNames) {
        let loc = new LinesAndColumns(this.input).locationForIndex(pos);
        if (!loc) {
            loc = { line: 0, column: 0 };
        }
        message += ` (${loc.line}:${loc.column})`;
        // tslint:disable-next-line no-any
        const err = new SyntaxError(message);
        err.pos = pos;
        err.loc = loc;
        if (missingPluginNames) {
            // @ts-ignore
            err.missingPlugin = missingPluginNames;
        }
        throw err;
    }
}
