import { IdentifierRole } from "../tokenizer";
import { types as tt } from "../tokenizer/types";
import UtilParser from "./util";
export default class LValParser extends UtilParser {
    // Parses spread element.
    parseSpread() {
        this.next();
        this.parseMaybeAssign(false);
    }
    parseRest(isBlockScope) {
        this.next();
        this.parseBindingAtom(isBlockScope);
    }
    parseBindingIdentifier() {
        this.parseIdentifier();
    }
    // Parses lvalue (assignable) atom.
    parseBindingAtom(isBlockScope) {
        switch (this.state.type) {
            case tt._yield:
            case tt.name: {
                this.state.type = tt.name;
                this.parseBindingIdentifier();
                this.state.tokens[this.state.tokens.length - 1].identifierRole = isBlockScope
                    ? IdentifierRole.BlockScopedDeclaration
                    : IdentifierRole.FunctionScopedDeclaration;
                return;
            }
            case tt.bracketL: {
                this.next();
                this.parseBindingList(tt.bracketR, isBlockScope, true /* allowEmpty */);
                return;
            }
            case tt.braceL:
                this.parseObj(true, isBlockScope);
                return;
            default:
                throw this.unexpected();
        }
    }
    parseBindingList(close, isBlockScope, allowEmpty, allowModifiers = null) {
        let first = true;
        let hasRemovedComma = false;
        const firstItemTokenIndex = this.state.tokens.length;
        while (!this.eat(close)) {
            if (first) {
                first = false;
            }
            else {
                this.expect(tt.comma);
                // After a "this" type in TypeScript, we need to set the following comma (if any) to also be
                // a type token so that it will be removed.
                if (!hasRemovedComma && this.state.tokens[firstItemTokenIndex].isType) {
                    this.state.tokens[this.state.tokens.length - 1].isType = true;
                    hasRemovedComma = true;
                }
            }
            if (allowEmpty && this.match(tt.comma)) {
                // Empty item; nothing further to parse for this item.
            }
            else if (this.eat(close)) {
                break;
            }
            else if (this.match(tt.ellipsis)) {
                this.parseRest(isBlockScope);
                this.parseAssignableListItemTypes();
                this.expect(close);
                break;
            }
            else {
                this.parseAssignableListItem(allowModifiers, isBlockScope);
            }
        }
    }
    parseAssignableListItem(allowModifiers, isBlockScope) {
        this.parseMaybeDefault(isBlockScope);
        this.parseAssignableListItemTypes();
        this.parseMaybeDefault(isBlockScope, true /* leftAlreadyParsed */);
    }
    parseAssignableListItemTypes() { }
    // Parses assignment pattern around given atom if possible.
    parseMaybeDefault(isBlockScope, leftAlreadyParsed = false) {
        if (!leftAlreadyParsed) {
            this.parseBindingAtom(isBlockScope);
        }
        if (!this.eat(tt.eq)) {
            return;
        }
        this.parseMaybeAssign();
    }
}
