"use strict";
/* eslint max-len: 0 */
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../tokenizer/types");
const types_2 = require("./types");
// tslint:disable-next-line no-any
function isMaybeDefaultImport(lookahead) {
    return (lookahead.type === types_1.types.name || !!lookahead.type.keyword) && lookahead.value !== "from";
}
class FlowParser extends types_2.default {
    flowParseTypeInitialiser(tok) {
        this.runInTypeContext(0, () => {
            this.expect(tok || types_1.types.colon);
            this.flowParseType();
        });
    }
    flowParsePredicate() {
        this.expect(types_1.types.modulo);
        this.expectContextual("checks");
        if (this.eat(types_1.types.parenL)) {
            this.parseExpression();
            this.expect(types_1.types.parenR);
        }
    }
    flowParseTypeAndPredicateInitialiser() {
        this.runInTypeContext(0, () => {
            this.expect(types_1.types.colon);
            if (this.match(types_1.types.modulo)) {
                this.flowParsePredicate();
            }
            else {
                this.flowParseType();
                if (this.match(types_1.types.modulo)) {
                    this.flowParsePredicate();
                }
            }
        });
    }
    flowParseDeclareClass() {
        this.next();
        this.flowParseInterfaceish(/* isClass */ true);
    }
    flowParseDeclareFunction() {
        this.next();
        this.parseIdentifier();
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        this.expect(types_1.types.parenL);
        this.flowParseFunctionTypeParams();
        this.expect(types_1.types.parenR);
        this.flowParseTypeAndPredicateInitialiser();
        this.semicolon();
    }
    flowParseDeclare() {
        if (this.match(types_1.types._class)) {
            this.flowParseDeclareClass();
        }
        else if (this.match(types_1.types._function)) {
            this.flowParseDeclareFunction();
        }
        else if (this.match(types_1.types._var)) {
            this.flowParseDeclareVariable();
        }
        else if (this.isContextual("module")) {
            if (this.lookaheadType() === types_1.types.dot) {
                this.flowParseDeclareModuleExports();
            }
            else {
                this.flowParseDeclareModule();
            }
        }
        else if (this.isContextual("type")) {
            this.flowParseDeclareTypeAlias();
        }
        else if (this.isContextual("opaque")) {
            this.flowParseDeclareOpaqueType();
        }
        else if (this.isContextual("interface")) {
            this.flowParseDeclareInterface();
        }
        else if (this.match(types_1.types._export)) {
            this.flowParseDeclareExportDeclaration();
        }
        else {
            throw this.unexpected();
        }
    }
    flowParseDeclareVariable() {
        this.next();
        this.flowParseTypeAnnotatableIdentifier();
        this.semicolon();
    }
    flowParseDeclareModule() {
        this.next();
        if (this.match(types_1.types.string)) {
            this.parseExprAtom();
        }
        else {
            this.parseIdentifier();
        }
        this.expect(types_1.types.braceL);
        while (!this.match(types_1.types.braceR)) {
            if (this.match(types_1.types._import)) {
                this.next();
                this.parseImport();
            }
        }
        this.expect(types_1.types.braceR);
    }
    flowParseDeclareExportDeclaration() {
        this.expect(types_1.types._export);
        if (this.eat(types_1.types._default)) {
            if (this.match(types_1.types._function) || this.match(types_1.types._class)) {
                // declare export default class ...
                // declare export default function ...
                this.flowParseDeclare();
            }
            else {
                // declare export default [type];
                this.flowParseType();
                this.semicolon();
            }
        }
        else if (this.match(types_1.types._var) || // declare export var ...
            this.match(types_1.types._function) || // declare export function ...
            this.match(types_1.types._class) || // declare export class ...
            this.isContextual("opaque") // declare export opaque ..
        ) {
            this.flowParseDeclare();
        }
        else if (this.match(types_1.types.star) || // declare export * from ''
            this.match(types_1.types.braceL) || // declare export {} ...
            this.isContextual("interface") || // declare export interface ...
            this.isContextual("type") || // declare export type ...
            this.isContextual("opaque") // declare export opaque type ...
        ) {
            this.parseExport();
        }
        else {
            throw this.unexpected();
        }
    }
    flowParseDeclareModuleExports() {
        this.expectContextual("module");
        this.expect(types_1.types.dot);
        this.expectContextual("exports");
        this.parseTypeAnnotation();
        this.semicolon();
    }
    flowParseDeclareTypeAlias() {
        this.next();
        this.flowParseTypeAlias();
    }
    flowParseDeclareOpaqueType() {
        this.next();
        this.flowParseOpaqueType(true);
    }
    flowParseDeclareInterface() {
        this.next();
        this.flowParseInterfaceish();
    }
    // Interfaces
    flowParseInterfaceish(isClass) {
        this.flowParseRestrictedIdentifier();
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        if (this.eat(types_1.types._extends)) {
            do {
                this.flowParseInterfaceExtends();
            } while (!isClass && this.eat(types_1.types.comma));
        }
        if (this.isContextual("mixins")) {
            this.next();
            do {
                this.flowParseInterfaceExtends();
            } while (this.eat(types_1.types.comma));
        }
        this.flowParseObjectType(true, false);
    }
    flowParseInterfaceExtends() {
        this.flowParseQualifiedTypeIdentifier(false);
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterInstantiation();
        }
    }
    flowParseInterface() {
        this.flowParseInterfaceish();
    }
    flowParseRestrictedIdentifier() {
        this.parseIdentifier();
    }
    // Type aliases
    flowParseTypeAlias() {
        this.flowParseRestrictedIdentifier();
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        this.flowParseTypeInitialiser(types_1.types.eq);
        this.semicolon();
    }
    flowParseOpaqueType(declare) {
        this.expectContextual("type");
        this.flowParseRestrictedIdentifier();
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        // Parse the supertype
        if (this.match(types_1.types.colon)) {
            this.flowParseTypeInitialiser(types_1.types.colon);
        }
        if (!declare) {
            this.flowParseTypeInitialiser(types_1.types.eq);
        }
        this.semicolon();
    }
    // Type annotations
    flowParseTypeParameter() {
        this.flowParseVariance();
        this.flowParseTypeAnnotatableIdentifier();
        if (this.eat(types_1.types.eq)) {
            this.flowParseType();
        }
    }
    flowParseTypeParameterDeclaration() {
        this.runInTypeContext(0, () => {
            // istanbul ignore else: this condition is already checked at all call sites
            if (this.match(types_1.types.lessThan) || this.match(types_1.types.typeParameterStart)) {
                this.next();
            }
            else {
                this.unexpected();
            }
            do {
                this.flowParseTypeParameter();
                if (!this.match(types_1.types.greaterThan)) {
                    this.expect(types_1.types.comma);
                }
            } while (!this.match(types_1.types.greaterThan));
            this.expect(types_1.types.greaterThan);
        });
    }
    flowParseTypeParameterInstantiation() {
        this.expect(types_1.types.lessThan);
        while (!this.match(types_1.types.greaterThan)) {
            this.flowParseType();
            if (!this.match(types_1.types.greaterThan)) {
                this.expect(types_1.types.comma);
            }
        }
        this.expect(types_1.types.greaterThan);
    }
    flowParseObjectPropertyKey() {
        if (this.match(types_1.types.num) || this.match(types_1.types.string)) {
            this.parseExprAtom();
        }
        else {
            this.parseIdentifier();
        }
    }
    flowParseObjectTypeIndexer() {
        this.expect(types_1.types.bracketL);
        if (this.lookaheadType() === types_1.types.colon) {
            this.flowParseObjectPropertyKey();
            this.flowParseTypeInitialiser();
        }
        else {
            this.flowParseType();
        }
        this.expect(types_1.types.bracketR);
        this.flowParseTypeInitialiser();
    }
    flowParseObjectTypeMethodish() {
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        this.expect(types_1.types.parenL);
        while (!this.match(types_1.types.parenR) && !this.match(types_1.types.ellipsis)) {
            this.flowParseFunctionTypeParam();
            if (!this.match(types_1.types.parenR)) {
                this.expect(types_1.types.comma);
            }
        }
        if (this.eat(types_1.types.ellipsis)) {
            this.flowParseFunctionTypeParam();
        }
        this.expect(types_1.types.parenR);
        this.flowParseTypeInitialiser();
    }
    flowParseObjectTypeCallProperty() {
        this.flowParseObjectTypeMethodish();
    }
    flowParseObjectType(allowStatic, allowExact) {
        let endDelim;
        if (allowExact && this.match(types_1.types.braceBarL)) {
            this.expect(types_1.types.braceBarL);
            endDelim = types_1.types.braceBarR;
        }
        else {
            this.expect(types_1.types.braceL);
            endDelim = types_1.types.braceR;
        }
        while (!this.match(endDelim)) {
            let isStatic = false;
            if (allowStatic && this.isContextual("static") && this.lookaheadType() !== types_1.types.colon) {
                this.next();
                isStatic = true;
            }
            this.flowParseVariance();
            if (this.match(types_1.types.bracketL)) {
                this.flowParseObjectTypeIndexer();
            }
            else if (this.match(types_1.types.parenL) || this.match(types_1.types.lessThan)) {
                this.flowParseObjectTypeCallProperty();
            }
            else {
                if (this.isContextual("get") || this.isContextual("set")) {
                    const lookaheadType = this.lookaheadType();
                    if (lookaheadType === types_1.types.name ||
                        lookaheadType === types_1.types.string ||
                        lookaheadType === types_1.types.num) {
                        this.next();
                    }
                }
                this.flowParseObjectTypeProperty();
            }
            this.flowObjectTypeSemicolon();
        }
        this.expect(endDelim);
    }
    flowParseObjectTypeProperty() {
        if (this.match(types_1.types.ellipsis)) {
            this.expect(types_1.types.ellipsis);
            this.flowParseType();
        }
        else {
            this.flowParseObjectPropertyKey();
            if (this.match(types_1.types.lessThan) || this.match(types_1.types.parenL)) {
                // This is a method property
                this.flowParseObjectTypeMethodish();
            }
            else {
                this.eat(types_1.types.question);
                this.flowParseTypeInitialiser();
            }
        }
    }
    flowObjectTypeSemicolon() {
        if (!this.eat(types_1.types.semi) &&
            !this.eat(types_1.types.comma) &&
            !this.match(types_1.types.braceR) &&
            !this.match(types_1.types.braceBarR)) {
            this.unexpected();
        }
    }
    flowParseQualifiedTypeIdentifier(initialIdAlreadyParsed) {
        if (!initialIdAlreadyParsed) {
            this.parseIdentifier();
        }
        while (this.eat(types_1.types.dot)) {
            this.parseIdentifier();
        }
    }
    flowParseGenericType() {
        this.flowParseQualifiedTypeIdentifier(true);
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterInstantiation();
        }
    }
    flowParseTypeofType() {
        this.expect(types_1.types._typeof);
        this.flowParsePrimaryType();
    }
    flowParseTupleType() {
        this.expect(types_1.types.bracketL);
        // We allow trailing commas
        while (this.state.pos < this.input.length && !this.match(types_1.types.bracketR)) {
            this.flowParseType();
            if (this.match(types_1.types.bracketR)) {
                break;
            }
            this.expect(types_1.types.comma);
        }
        this.expect(types_1.types.bracketR);
    }
    flowParseFunctionTypeParam() {
        const lookaheadType = this.lookaheadType();
        if (lookaheadType === types_1.types.colon || lookaheadType === types_1.types.question) {
            this.parseIdentifier();
            this.eat(types_1.types.question);
            this.flowParseTypeInitialiser();
        }
        else {
            this.flowParseType();
        }
    }
    flowParseFunctionTypeParams() {
        while (!this.match(types_1.types.parenR) && !this.match(types_1.types.ellipsis)) {
            this.flowParseFunctionTypeParam();
            if (!this.match(types_1.types.parenR)) {
                this.expect(types_1.types.comma);
            }
        }
        if (this.eat(types_1.types.ellipsis)) {
            this.flowParseFunctionTypeParam();
        }
    }
    flowIdentToTypeAnnotation(name) {
        switch (name) {
            case "any":
            case "void":
            case "bool":
            case "boolean":
            case "mixed":
            case "empty":
            case "number":
            case "string":
                return;
            default:
                this.flowParseGenericType();
        }
    }
    // The parsing of types roughly parallels the parsing of expressions, and
    // primary types are kind of like primary expressions...they're the
    // primitives with which other types are constructed.
    flowParsePrimaryType() {
        let isGroupedType = false;
        const oldNoAnonFunctionType = this.state.noAnonFunctionType;
        switch (this.state.type) {
            case types_1.types.name: {
                this.parseIdentifier();
                const name = this.state.tokens[this.state.tokens.length - 1].value;
                this.flowIdentToTypeAnnotation(name);
                return;
            }
            case types_1.types.braceL:
                this.flowParseObjectType(false, false);
                return;
            case types_1.types.braceBarL:
                this.flowParseObjectType(false, true);
                return;
            case types_1.types.bracketL:
                this.flowParseTupleType();
                return;
            case types_1.types.lessThan:
                this.flowParseTypeParameterDeclaration();
                this.expect(types_1.types.parenL);
                this.flowParseFunctionTypeParams();
                this.expect(types_1.types.parenR);
                this.expect(types_1.types.arrow);
                this.flowParseType();
                return;
            case types_1.types.parenL:
                this.next();
                // Check to see if this is actually a grouped type
                if (!this.match(types_1.types.parenR) && !this.match(types_1.types.ellipsis)) {
                    if (this.match(types_1.types.name)) {
                        const token = this.lookaheadType();
                        isGroupedType = token !== types_1.types.question && token !== types_1.types.colon;
                    }
                    else {
                        isGroupedType = true;
                    }
                }
                if (isGroupedType) {
                    this.state.noAnonFunctionType = false;
                    this.flowParseType();
                    this.state.noAnonFunctionType = oldNoAnonFunctionType;
                    // A `,` or a `) =>` means this is an anonymous function type
                    if (this.state.noAnonFunctionType ||
                        !(this.match(types_1.types.comma) || (this.match(types_1.types.parenR) && this.lookaheadType() === types_1.types.arrow))) {
                        this.expect(types_1.types.parenR);
                        return;
                    }
                    else {
                        // Eat a comma if there is one
                        this.eat(types_1.types.comma);
                    }
                }
                this.flowParseFunctionTypeParams();
                this.expect(types_1.types.parenR);
                this.expect(types_1.types.arrow);
                this.flowParseType();
                return;
            case types_1.types.plusMin:
                // Only allow negative signs, not plus signs.
                if (this.state.value === "-") {
                    this.next();
                    this.parseLiteral();
                    return;
                }
                throw this.unexpected();
            case types_1.types.string:
            case types_1.types.num:
            case types_1.types._true:
            case types_1.types._false:
            case types_1.types._null:
            case types_1.types._this:
            case types_1.types._void:
            case types_1.types.star:
                this.next();
                return;
            default:
                if (this.state.type.keyword === "typeof") {
                    this.flowParseTypeofType();
                    return;
                }
        }
        throw this.unexpected();
    }
    flowParsePostfixType() {
        this.flowParsePrimaryType();
        while (!this.canInsertSemicolon() && this.match(types_1.types.bracketL)) {
            this.expect(types_1.types.bracketL);
            this.expect(types_1.types.bracketR);
        }
    }
    flowParsePrefixType() {
        if (this.eat(types_1.types.question)) {
            this.flowParsePrefixType();
        }
        else {
            this.flowParsePostfixType();
        }
    }
    flowParseAnonFunctionWithoutParens() {
        this.flowParsePrefixType();
        if (!this.state.noAnonFunctionType && this.eat(types_1.types.arrow)) {
            this.flowParseType();
        }
    }
    flowParseIntersectionType() {
        this.eat(types_1.types.bitwiseAND);
        this.flowParseAnonFunctionWithoutParens();
        while (this.eat(types_1.types.bitwiseAND)) {
            this.flowParseAnonFunctionWithoutParens();
        }
    }
    flowParseUnionType() {
        this.eat(types_1.types.bitwiseOR);
        this.flowParseIntersectionType();
        while (this.eat(types_1.types.bitwiseOR)) {
            this.flowParseIntersectionType();
        }
    }
    flowParseType() {
        this.flowParseUnionType();
    }
    parseTypeAnnotation() {
        this.flowParseTypeInitialiser();
    }
    flowParseTypeAnnotatableIdentifier() {
        this.parseIdentifier();
        if (this.match(types_1.types.colon)) {
            this.parseTypeAnnotation();
        }
    }
    flowParseVariance() {
        if (this.match(types_1.types.plusMin)) {
            this.next();
        }
    }
    // ==================================
    // Overrides
    // ==================================
    parseFunctionBodyAndFinish(functionStart, isGenerator, allowExpressionBody, funcContextId) {
        // For arrow functions, `parseArrow` handles the return type itself.
        if (!allowExpressionBody && this.match(types_1.types.colon)) {
            this.flowParseTypeAndPredicateInitialiser();
        }
        super.parseFunctionBodyAndFinish(functionStart, isGenerator, allowExpressionBody, funcContextId);
    }
    // interfaces
    parseStatement(declaration, topLevel) {
        if (this.match(types_1.types.name) && this.state.value === "interface") {
            this.runInTypeContext(0, () => {
                this.next();
                this.flowParseInterface();
            });
        }
        else {
            super.parseStatement(declaration, topLevel);
        }
    }
    // declares, interfaces and type aliases
    parseIdentifierStatement(name) {
        if (name === "declare") {
            if (this.match(types_1.types._class) ||
                this.match(types_1.types.name) ||
                this.match(types_1.types._function) ||
                this.match(types_1.types._var) ||
                this.match(types_1.types._export)) {
                this.runInTypeContext(1, () => {
                    this.flowParseDeclare();
                });
            }
        }
        else if (this.match(types_1.types.name)) {
            if (name === "interface") {
                this.runInTypeContext(1, () => {
                    this.flowParseInterface();
                });
            }
            else if (name === "type") {
                this.runInTypeContext(1, () => {
                    this.flowParseTypeAlias();
                });
            }
            else if (name === "opaque") {
                this.runInTypeContext(1, () => {
                    this.flowParseOpaqueType(false);
                });
            }
        }
        super.parseIdentifierStatement(name);
    }
    // export type
    shouldParseExportDeclaration() {
        return (this.isContextual("type") ||
            this.isContextual("interface") ||
            this.isContextual("opaque") ||
            super.shouldParseExportDeclaration());
    }
    isExportDefaultSpecifier() {
        if (this.match(types_1.types.name) &&
            (this.state.value === "type" ||
                this.state.value === "interface" ||
                this.state.value === "opaque")) {
            return false;
        }
        return super.isExportDefaultSpecifier();
    }
    parseExportDeclaration() {
        if (this.isContextual("type")) {
            this.runInTypeContext(1, () => {
                this.next();
                if (this.match(types_1.types.braceL)) {
                    // export type { foo, bar };
                    this.parseExportSpecifiers();
                    this.parseExportFrom();
                }
                else {
                    // export type Foo = Bar;
                    this.flowParseTypeAlias();
                }
            });
        }
        else if (this.isContextual("opaque")) {
            this.runInTypeContext(1, () => {
                this.next();
                // export opaque type Foo = Bar;
                this.flowParseOpaqueType(false);
            });
        }
        else if (this.isContextual("interface")) {
            this.runInTypeContext(1, () => {
                this.next();
                this.flowParseInterface();
            });
        }
        else {
            super.parseExportDeclaration();
        }
    }
    shouldParseExportStar() {
        return (super.shouldParseExportStar() ||
            (this.isContextual("type") && this.lookaheadType() === types_1.types.star));
    }
    parseExportStar() {
        if (this.eatContextual("type")) {
            this.runInTypeContext(2, () => {
                super.parseExportStar();
            });
        }
        else {
            super.parseExportStar();
        }
    }
    parseClassId(isStatement, optionalId = false) {
        super.parseClassId(isStatement, optionalId);
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
    }
    // parse an item inside a expression list eg. `(NODE, NODE)` where NODE represents
    // the position where this function is called
    parseExprListItem(allowEmpty) {
        super.parseExprListItem(allowEmpty);
        if (this.match(types_1.types.colon)) {
            this.parseTypeAnnotation();
        }
    }
    // parse class property type annotations
    parseClassProperty() {
        if (this.match(types_1.types.colon)) {
            this.parseTypeAnnotation();
        }
        super.parseClassProperty();
    }
    // parse type parameters for class methods
    parseClassMethod(functionStart, isGenerator, isConstructor) {
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
        }
        super.parseClassMethod(functionStart, isGenerator, isConstructor);
    }
    // parse a the super class type parameters and implements
    parseClassSuper() {
        const hadSuper = super.parseClassSuper();
        if (hadSuper && this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterInstantiation();
        }
        if (this.isContextual("implements")) {
            this.state.tokens[this.state.tokens.length - 1].type = types_1.types._implements;
            this.runInTypeContext(0, () => {
                this.next();
                do {
                    this.flowParseRestrictedIdentifier();
                    if (this.match(types_1.types.lessThan)) {
                        this.flowParseTypeParameterInstantiation();
                    }
                } while (this.eat(types_1.types.comma));
            });
        }
        return hadSuper;
    }
    parsePropertyName(classContextId) {
        this.flowParseVariance();
        super.parsePropertyName(classContextId);
    }
    // parse type parameters for object method shorthand
    parseObjPropValue(isGenerator, isPattern, isBlockScope, objectContextId) {
        // method shorthand
        if (this.match(types_1.types.lessThan)) {
            this.flowParseTypeParameterDeclaration();
            if (!this.match(types_1.types.parenL))
                this.unexpected();
        }
        super.parseObjPropValue(isGenerator, isPattern, isBlockScope, objectContextId);
    }
    parseAssignableListItemTypes() {
        this.runInTypeContext(0, () => {
            this.eat(types_1.types.question);
            if (this.match(types_1.types.colon)) {
                this.parseTypeAnnotation();
            }
        });
    }
    // parse typeof and type imports
    parseImportSpecifiers() {
        let kind = null;
        if (this.match(types_1.types._typeof)) {
            kind = "typeof";
        }
        else if (this.isContextual("type")) {
            kind = "type";
        }
        if (kind) {
            const lh = this.lookaheadTypeAndValue();
            if (isMaybeDefaultImport(lh) || lh.type === types_1.types.braceL || lh.type === types_1.types.star) {
                this.next();
            }
        }
        super.parseImportSpecifiers();
    }
    // parse import-type/typeof shorthand
    parseImportSpecifier() {
        this.parseIdentifier();
        const firstIdentName = this.state.tokens[this.state.tokens.length - 1].value;
        let specifierTypeKind = null;
        if (firstIdentName === "type") {
            this.state.tokens[this.state.tokens.length - 1].type = types_1.types._type;
            specifierTypeKind = "type";
        }
        else if (firstIdentName === "typeof") {
            specifierTypeKind = "typeof";
            this.state.tokens[this.state.tokens.length - 1].type = types_1.types._typeof;
        }
        if (this.isContextual("as") && !this.isLookaheadContextual("as")) {
            this.parseIdentifier();
            if (specifierTypeKind !== null && !this.match(types_1.types.name) && !this.state.type.keyword) {
                // `import {type as ,` or `import {type as }`
            }
            else {
                // `import {type as foo`
                this.parseIdentifier();
            }
        }
        else if (specifierTypeKind !== null && (this.match(types_1.types.name) || this.state.type.keyword)) {
            // `import {type foo`
            this.parseIdentifier();
            if (this.eatContextual("as")) {
                this.parseIdentifier();
            }
        }
    }
    // parse function type parameters - function foo<T>() {}
    parseFunctionParams(allowModifiers, contextId) {
        // Originally this checked if the method is a getter/setter, but if it was, we'd crash soon
        // anyway, so don't try to propagate that information.
        if (this.match(types_1.types.lessThan)) {
            this.runInTypeContext(0, () => {
                this.flowParseTypeParameterDeclaration();
            });
        }
        super.parseFunctionParams(allowModifiers, contextId);
    }
    // parse flow type annotations on variable declarator heads - let foo: string = bar
    parseVarHead(isBlockScope) {
        super.parseVarHead(isBlockScope);
        if (this.match(types_1.types.colon)) {
            this.parseTypeAnnotation();
        }
    }
    // parse the return type of an async arrow function - let foo = (async (): number => {});
    parseAsyncArrowFromCallExpression(functionStart, startTokenIndex) {
        if (this.match(types_1.types.colon)) {
            const oldNoAnonFunctionType = this.state.noAnonFunctionType;
            this.state.noAnonFunctionType = true;
            this.parseTypeAnnotation();
            this.state.noAnonFunctionType = oldNoAnonFunctionType;
        }
        super.parseAsyncArrowFromCallExpression(functionStart, startTokenIndex);
    }
    // We need to support type parameter declarations for arrow functions. This
    // is tricky. There are three situations we need to handle
    //
    // 1. This is either JSX or an arrow function. We'll try JSX first. If that
    //    fails, we'll try an arrow function. If that fails, we'll throw the JSX
    //    error.
    // 2. This is an arrow function. We'll parse the type parameter declaration,
    //    parse the rest, make sure the rest is an arrow function, and go from
    //    there
    // 3. This is neither. Just call the super method
    parseMaybeAssign(noIn, afterLeftParse) {
        let jsxError = null;
        if (types_1.types.jsxTagStart && this.match(types_1.types.jsxTagStart)) {
            const snapshot = this.state.snapshot();
            try {
                return super.parseMaybeAssign(noIn, afterLeftParse);
            }
            catch (err) {
                if (err instanceof SyntaxError) {
                    this.state.restoreFromSnapshot(snapshot);
                    this.state.type = types_1.types.typeParameterStart;
                    jsxError = err;
                }
                else {
                    // istanbul ignore next: no such error is expected
                    throw err;
                }
            }
        }
        if (jsxError != null || this.match(types_1.types.lessThan)) {
            let wasArrow = false;
            try {
                this.runInTypeContext(0, () => {
                    this.flowParseTypeParameterDeclaration();
                });
                wasArrow = super.parseMaybeAssign(noIn, afterLeftParse);
            }
            catch (err) {
                throw jsxError || err;
            }
            if (wasArrow) {
                return true;
            }
            this.unexpected();
        }
        return super.parseMaybeAssign(noIn, afterLeftParse);
    }
    // handle return types for arrow functions
    parseArrow() {
        if (this.match(types_1.types.colon)) {
            this.runInTypeContext(0, () => {
                const snapshot = this.state.snapshot();
                try {
                    const oldNoAnonFunctionType = this.state.noAnonFunctionType;
                    this.state.noAnonFunctionType = true;
                    this.flowParseTypeAndPredicateInitialiser();
                    this.state.noAnonFunctionType = oldNoAnonFunctionType;
                    if (this.canInsertSemicolon())
                        this.unexpected();
                    if (!this.match(types_1.types.arrow))
                        this.unexpected();
                }
                catch (err) {
                    if (err instanceof SyntaxError) {
                        this.state.restoreFromSnapshot(snapshot);
                    }
                    else {
                        // istanbul ignore next: no such error is expected
                        throw err;
                    }
                }
            });
        }
        return super.parseArrow();
    }
    parseSubscripts(startPos, noCalls) {
        if (this.state.tokens[this.state.tokens.length - 1].value === "async" &&
            this.match(types_1.types.lessThan)) {
            const snapshot = this.state.snapshot();
            let error;
            try {
                const wasArrow = this.parseAsyncArrowWithTypeParameters(startPos);
                if (wasArrow) {
                    return;
                }
            }
            catch (e) {
                error = e;
            }
            this.state.restoreFromSnapshot(snapshot);
            try {
                super.parseSubscripts(startPos, noCalls);
                return;
            }
            catch (e) {
                throw error || e;
            }
        }
        super.parseSubscripts(startPos, noCalls);
    }
    // Returns true if there was an arrow function here.
    parseAsyncArrowWithTypeParameters(startPos) {
        const startTokenIndex = this.state.tokens.length;
        this.parseFunctionParams();
        if (!this.parseArrow()) {
            return false;
        }
        this.parseArrowExpression(startPos, startTokenIndex);
        return true;
    }
}
exports.default = FlowParser;
