"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../tokenizer/types");
const jsx_1 = require("./jsx");
/**
 * Common parser code for TypeScript and Flow.
 */
class TypeParser extends jsx_1.default {
    // An apparent conditional expression could actually be an optional parameter in an arrow function.
    parseConditional(noIn, startPos) {
        // only do the expensive clone if there is a question mark
        // and if we come from inside parens
        if (!this.match(types_1.types.question)) {
            super.parseConditional(noIn, startPos);
            return;
        }
        const snapshot = this.state.snapshot();
        try {
            super.parseConditional(noIn, startPos);
            return;
        }
        catch (err) {
            if (!(err instanceof SyntaxError)) {
                // istanbul ignore next: no such error is expected
                throw err;
            }
            this.state.restoreFromSnapshot(snapshot);
        }
    }
    // Note: These "type casts" are *not* valid TS expressions.
    // But we parse them here and change them when completing the arrow function.
    parseParenItem() {
        super.parseParenItem();
        if (this.eat(types_1.types.question)) {
            this.state.tokens[this.state.tokens.length - 1].isType = true;
        }
        if (this.match(types_1.types.colon)) {
            this.parseTypeAnnotation();
        }
    }
    isClassMethod() {
        return this.match(types_1.types.lessThan) || super.isClassMethod();
    }
    isClassProperty() {
        return this.match(types_1.types.colon) || super.isClassProperty();
    }
    shouldParseArrow() {
        return this.match(types_1.types.colon) || super.shouldParseArrow();
    }
    shouldParseAsyncArrow() {
        return this.match(types_1.types.colon) || super.shouldParseAsyncArrow();
    }
}
exports.default = TypeParser;
