import { types as tt } from "../tokenizer/types";
import JSXParser from "./jsx";
/**
 * Common parser code for TypeScript and Flow.
 */
export default class TypeParser extends JSXParser {
    // An apparent conditional expression could actually be an optional parameter in an arrow function.
    parseConditional(noIn, startPos) {
        // only do the expensive clone if there is a question mark
        // and if we come from inside parens
        if (!this.match(tt.question)) {
            super.parseConditional(noIn, startPos);
            return;
        }
        const snapshot = this.state.snapshot();
        try {
            super.parseConditional(noIn, startPos);
            return;
        }
        catch (err) {
            if (!(err instanceof SyntaxError)) {
                // istanbul ignore next: no such error is expected
                throw err;
            }
            this.state.restoreFromSnapshot(snapshot);
        }
    }
    // Note: These "type casts" are *not* valid TS expressions.
    // But we parse them here and change them when completing the arrow function.
    parseParenItem() {
        super.parseParenItem();
        if (this.eat(tt.question)) {
            this.state.tokens[this.state.tokens.length - 1].isType = true;
        }
        if (this.match(tt.colon)) {
            this.parseTypeAnnotation();
        }
    }
    isClassMethod() {
        return this.match(tt.lessThan) || super.isClassMethod();
    }
    isClassProperty() {
        return this.match(tt.colon) || super.isClassProperty();
    }
    shouldParseArrow() {
        return this.match(tt.colon) || super.shouldParseArrow();
    }
    shouldParseAsyncArrow() {
        return this.match(tt.colon) || super.shouldParseAsyncArrow();
    }
}
