import { TokenType } from "../tokenizer/types";
import TypeParser from "./types";
export declare type TsModifier = "readonly" | "abstract" | "static" | "public" | "private" | "protected";
export declare type ParsingContext = "EnumMembers" | "HeritageClauseElement" | "TupleElementTypes" | "TypeMembers" | "TypeParametersOrArguments";
export default class TypeScriptParser extends TypeParser {
    tsIsIdentifier(): boolean;
    tsNextTokenCanFollowModifier(): boolean;
    /** Parses a modifier matching one the given modifier names. */
    tsParseModifier<T extends TsModifier>(allowedModifiers: Array<T>): T | null;
    tsIsListTerminator(kind: ParsingContext): boolean;
    tsParseList(kind: ParsingContext, parseElement: () => void): void;
    tsParseDelimitedList(kind: ParsingContext, parseElement: () => void): void;
    /**
     * If !expectSuccess, returns undefined instead of failing to parse.
     * If expectSuccess, parseElement should always return a defined value.
     */
    tsParseDelimitedListWorker(kind: ParsingContext, parseElement: () => void): void;
    tsParseBracketedList(kind: ParsingContext, parseElement: () => void, bracket: boolean, skipFirstToken: boolean): void;
    tsParseEntityName(): void;
    tsParseTypeReference(): void;
    tsParseThisTypePredicate(): void;
    tsParseThisTypeNode(): void;
    tsParseTypeQuery(): void;
    tsParseTypeParameter(): void;
    tsTryParseTypeParameters(): void;
    tsParseTypeParameters(): void;
    tsFillSignature(returnToken: TokenType): void;
    tsParseBindingListForSignature(isBlockScope: boolean): void;
    tsParseTypeMemberSemicolon(): void;
    tsParseSignatureMember(kind: "TSCallSignatureDeclaration" | "TSConstructSignatureDeclaration"): void;
    tsIsUnambiguouslyIndexSignature(): boolean;
    tsTryParseIndexSignature(): boolean;
    tsParsePropertyOrMethodSignature(readonly: boolean): void;
    tsParseTypeMember(): void;
    tsIsStartOfConstructSignature(): boolean;
    tsParseTypeLiteral(): void;
    tsParseObjectTypeMembers(): void;
    tsIsStartOfMappedType(): boolean;
    tsParseMappedTypeParameter(): void;
    tsParseMappedType(): void;
    tsParseTupleType(): void;
    tsParseParenthesizedType(): void;
    tsParseFunctionOrConstructorType(type: "TSFunctionType" | "TSConstructorType"): void;
    tsParseNonArrayType(): void;
    tsParseArrayTypeOrHigher(): void;
    tsParseTypeOperator(operator: "keyof"): void;
    tsParseTypeOperatorOrHigher(): void;
    tsParseUnionOrIntersectionType(kind: "TSUnionType" | "TSIntersectionType", parseConstituentType: () => void, operator: TokenType): void;
    tsParseIntersectionTypeOrHigher(): void;
    tsParseUnionTypeOrHigher(): void;
    tsIsStartOfFunctionType(): boolean;
    tsSkipParameterStart(): boolean;
    tsIsUnambiguouslyStartOfFunctionType(): boolean;
    tsParseTypeOrTypePredicateAnnotation(returnToken: TokenType): void;
    tsTryParseTypeOrTypePredicateAnnotation(): void;
    tsTryParseTypeAnnotation(): void;
    tsTryParseType(): void;
    tsParseTypePredicatePrefix(): boolean;
    parseTypeAnnotation(): void;
    tsParseType(): void;
    tsParseTypeAssertion(): void;
    tsTryParseTypeArgumentsInExpression(): boolean;
    tsParseHeritageClause(): void;
    tsParseExpressionWithTypeArguments(): void;
    tsParseInterfaceDeclaration(): void;
    tsParseTypeAliasDeclaration(): void;
    tsParseEnumMember(): void;
    tsParseEnumDeclaration(): void;
    tsParseModuleBlock(): void;
    tsParseModuleOrNamespaceDeclaration(): void;
    tsParseAmbientExternalModuleDeclaration(): void;
    tsParseImportEqualsDeclaration(): void;
    tsIsExternalModuleReference(): boolean;
    tsParseModuleReference(): void;
    tsParseExternalModuleReference(): void;
    tsLookAhead<T>(f: () => T): T;
    tsTryParseAndCatch<T>(f: () => void): boolean;
    tsTryParse<T>(f: () => boolean): boolean;
    tsTryParseDeclare(): boolean;
    tsTryParseExportDeclaration(): boolean;
    tsParseExpressionStatement(name: string): boolean;
    tsParseDeclaration(name: string, next: boolean): boolean;
    tsTryParseGenericAsyncArrowFunction(): boolean;
    tsParseTypeArguments(): void;
    tsIsDeclarationStart(): boolean;
    isExportDefaultSpecifier(): boolean;
    parseAssignableListItem(allowModifiers: boolean | null, isBlockScope: boolean): void;
    parseFunctionBodyAndFinish(functionStart: number, isGenerator: boolean, allowExpressionBody?: boolean, funcContextId?: number): void;
    parseSubscript(startPos: number, noCalls: boolean | null, state: {
        stop: boolean;
    }): void;
    parseNewArguments(): void;
    parseExprOp(minPrec: number, noIn: boolean | null): void;
    checkDuplicateExports(): void;
    parseImport(): void;
    parseExport(): void;
    parseExportDefaultExpression(): void;
    parseStatementContent(declaration: boolean, topLevel?: boolean): void;
    parseAccessModifier(): void;
    parseClassMember(memberStart: number, classContextId: number): void;
    parseClassMemberWithIsStatic(memberStart: number, isStatic: boolean, classContextId: number): void;
    parsePostMemberNameModifiers(): void;
    parseIdentifierStatement(name: string): void;
    shouldParseExportDeclaration(): boolean;
    parseExportDeclaration(): void;
    parseClassId(isStatement: boolean, optionalId?: boolean): void;
    parseClassProperty(): void;
    parseClassMethod(functionStart: number, isGenerator: boolean, isConstructor: boolean): void;
    parseClassSuper(): boolean;
    parseObjPropValue(isGenerator: boolean, isPattern: boolean, isBlockScope: boolean, objectContextId: number): void;
    parseFunctionParams(allowModifiers?: boolean, contextId?: number): void;
    parseVarHead(isBlockScope: boolean): void;
    parseAsyncArrowFromCallExpression(functionStart: number, startTokenIndex: number): void;
    parseMaybeAssign(noIn?: boolean | null, afterLeftParse?: Function): boolean;
    parseMaybeUnary(): boolean;
    parseArrow(): boolean;
    parseAssignableListItemTypes(): void;
    parseBindingAtom(isBlockScope: boolean): void;
}
